local internal = require "engine.networkingInternal"
local utils = require 'engine.utils'

-- Interface

-- networking extends (or hides methods from) networking.internal 
local networking = setmetatable({}, {
	__index = function(self, key)
		return internal[key]
	end
})

function networking.connect(forceServerSearch)
	if forceServerSearch ~= nil then 
		internal.connect(forceServerSearch)
	else
		internal.connect(false)
	end
end

function networking.startRandomGame(maxPlayers, customProperties)
	local keys, values = utils.toKeyAndValueArrays(customProperties)
	internal.startRandomGame(maxPlayers, keys, values)
end

function networking.startGameWithFriends(
			requestId,
			requestMessage,
			friendIds,
			maxPlayers,
			customProperties)

	local keys, values = utils.toKeyAndValueArrays(customProperties)

	internal.startGameWithFriends(
			requestId,
			requestMessage,
			friendIds,
			maxPlayers,
			keys,
			values)

end

-- Marks a sprite as a network object, with the given network object type
function networking.registerNetworkObject(sprite, networkObjectType)
	internal.registerNetworkObject(sprite.id, networkObjectType)
end

function networking.enableSyncOf(sprite, fieldId)
    internal.enableStateSync(sprite.id, fieldId)
end

function networking.disableSyncOf(sprite, fieldId)
    internal.disableStateSync(sprite.id, fieldId)
end

function networking.isLocal(sprite)
    return internal.isLocal(sprite.id)
end

function networking.isShadow(sprite)
    return internal.isShadow(sprite.id)
end

function networking.playerIdToFriendId(playerId)
	if (type(playerId) ~= "string") then
		error("playerId must be of type string, given: " .. type(playerId))
	end

	return internal.playerIdToFriendId(playerId)
end

return networking