local sc = require 'engine.social'
local gm = require 'engine.manager'

socialHelper = {}

--[[

    string.split (s, p)
    ====================================================================
    Splits the string [s] into substrings wherever pattern [p] occurs.

    Returns: a table of substrings or, if no match is made [nil].

--]]
string.split = function(s, p)
    local temp = {}
    local index = 0
    local last_index = string.len(s)

    while true do
        local i, e = string.find(s, p, index)

        if i and e then
            local next_index = e + 1
            local word_bound = i - 1
            table.insert(temp, string.sub(s, index, word_bound))
            index = next_index
        else            
            if index > 0 and index <= last_index then
                table.insert(temp, string.sub(s, index, last_index))
            elseif index == 0 then
                temp = nil
            end
            break
        end
    end

    return temp
end

function socialHelper.getSocialFriend(sprites, start_index, propertyQuery, textFrameQuery)
	local length = sprites.Length
	local friends = sc.getFriends(0, start_index, length)	
	local len = math.min(length, #friends)

	for i=1, len do
		for prop, socialParam in pairs(propertyQuery) do
			local s = sprites[i]
			local parts = string.split(prop, "%.")

			if #parts == 2 then
				s[parts[1]][parts[2]] = friends[i][socialParam]
			elseif #parts == 1 then
				s[parts[1]] = friends[i][socialParam]
			else
				error('socialHelper.getSocialFriend() invalid number of parts given ' .. #parts) 
			end
		end
			
		for frame, socialParam in pairs(textFrameQuery) do
			
			local s = sprites[i]
			local currParam = friends[i][socialParam]

			if currParam then
				s.canvas[frame].text = tostring(currParam)
			else
				s.canvas[frame].text = ""
			end
		end	
	end
end

function socialHelper.displayImage(sprites, start_index, toCrop)
	
	local length = sprites.Length
	local friendsImages = sc.getFriendsImages(0, start_index, length)
	
	length = math.min(length, #friendsImages)

	for i=1, length do
		local s = sprites[i]
		local image  = friendsImages[i]
		
		gm.setImage(s.id, image.AnimationId, toCrop)

	end
end

function socialHelper.submitHighscore(score)

	sc.submitHighscore(score)
end

function socialHelper.loadFriends(sprites, eventName, usersType, os, isScoreMoreThanZero, isRelativeToMe, start_index, count, sortBy, direction, friendId, isForcedLoad)
	--[[ 
		socialHelper_loadFriends_callback
		is a global variable - so the GC won't take it while we have a reference to it in the C#/Java code.
		This causes for only one function availble at a time - meaning EVERY load action will replace the lua function
		with a new one. This is quite good for one availble list - because every load creates a new list, and allows only one
		handler function.
		!!! WHEN CHANGING TO MULTIPLE LISTS - THIS CODE MUST BE CHANGED TO ALLOW IT !!!
	--]]
	socialHelper_loadFriends_callback = function (args)
		for i=1, sprites.Length do
			sprites[i][eventName](args)
		end
	end

	if friendId == nil then
		friendId = -1
	end

	sc.loadFriends("socialHelper_loadFriends_callback", usersType, os, isScoreMoreThanZero, isRelativeToMe, start_index, count, sortBy, direction, friendId, isForcedLoad)
end

function socialHelper.sendRequest(friendId, requestId, message, quantity, arg1, arg2, arg3)

	if not quantity then
		quantity = 0
	end
	if not arg1 then
		arg1 = 0
	end
	if not arg2 then
		arg2 = 0
	end
	if not arg3 then
		arg3 = 0
	end

	sc.sendRequest( { friendId }, requestId, message, quantity, arg1, arg2, arg3)
end

function socialHelper.checkRequests(sprites, onSuccessEvent, onFailEvent)
	local onSuccessCallbackName = "socialHelper_checkRequests_onSuccessCallback"
	local onFailCallbackName = "socialHelper_checkRequests_onFailCallbackName"
    
	_G[onSuccessCallbackName] = function (args)
		local requests = args["requests"]
        len = sprites.Length
		for i=1, #requests do
			for j=1, len do
				sprites[j][onSuccessEvent](requests[i])
			end
		end
	end

	_G[onFailCallbackName] = function ()
        len = sprites.Length
		for i=1, len do
			sprites[i][onFailEvent]()
		end
	end

	sc.checkRequests(onSuccessCallbackName, onFailCallbackName)
end

return socialHelper